package cn.com.ctop.kuaishou.modules.batch.service.impl;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouProjectVideoGet;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouVideoGetMapper;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaishouProjectVideoGetMapper;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouImageGetService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouMaterialUploadService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouProjectVideoGetService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 快手-项目下视频信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-05-21
 */
@Service
public class KuaishouProjectVideoGetServiceImpl extends ServiceImpl<KuaishouProjectVideoGetMapper, KuaishouProjectVideoGet> implements IKuaishouProjectVideoGetService {

    @Autowired
    private KuaishouProjectVideoGetMapper projectVideoGetMapper;
    @Autowired
    private KuaiShouVideoGetMapper videoGetMapper;
    @Autowired
    private IKuaiShouImageGetService imageGetService;
    @Autowired
    private IKuaiShouMaterialUploadService uploadService;

    @Override
    public void clearProjectVideo(Long projectId) {
        String updateTime = projectVideoGetMapper.queryLatestUpdateTime(projectId);
        System.out.println(updateTime);
        List<KuaishouProjectVideoGet> list = videoGetMapper.selectProjectVideo(projectId, updateTime);
        if (!Check.isNull(list)) {
            projectVideoGetMapper.replaceBatch(list);
        }
    }

    @Override
    public Result<Object> queryPageList(KuaishouProjectVideoGet kuaishouProjectVideoGet, Integer pageNo, Integer pageSize) {
        Long total = projectVideoGetMapper.queryPageListTotal(kuaishouProjectVideoGet);
        PageHelper.startPage(pageNo, pageSize, false);
        PageInfo pageInfo = new PageInfo(projectVideoGetMapper.queryPageList(kuaishouProjectVideoGet));
        pageInfo.setTotal(total);
        return Result.ok(pageInfo);
    }

    @Override
    public Result<Object> getVideoTotalNumber(Long projectId) {
        //本周一
        String monday = DateUtils.getdateFromWeek(1);
        //当天日期
        String today = DateUtils.formatDate(new Date());
        //上周一日期
        String lastStartDate = DateUtils.getLastDay(monday, 7);
        //上周当天日期
        String lastEndDate = DateUtils.getLastDay(today, 7);
        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("lastStartDate", lastStartDate);
        paramsMap.put("lastEndDate", lastEndDate);
        paramsMap.put("monday", monday);
        paramsMap.put("today", today);
        paramsMap.put("projectId", projectId);
        return Result.ok(projectVideoGetMapper.getVideoTotalNumber(paramsMap));
    }

    @Override
    public Result<Object> getVideoDetails(String signature) {
        return Result.ok(projectVideoGetMapper.getVideoDetails(signature));
    }

    @Override
    public Result<Object> getAccountVideos(String signature, Long projectId, Integer pageNo, Integer pageSize) {
        Long total = projectVideoGetMapper.getAccountVideosTotal(signature, projectId);
        PageHelper.startPage(pageNo, pageSize, false);
        PageInfo<JSONObject> list = new PageInfo<>(projectVideoGetMapper.getAccountVideos(signature, projectId));
        list.setTotal(total);
        return Result.ok(list);
    }

}
